-- QuestGlance by Astromech
-- Contact on WoWInterface

-- Edit preferences here:
-- Show the number of complete quests on the display
local showComplete = true
-- Show the maximum number of quests you can hold on the display
local showMax = false
-- Show daily quests in the tooltip
local showDailiesInTip = true
-- Show completed quests in the tooltip
local showCompletesInTip = true

-- Strings for future localization
local L_Click = "Click to toggle quest log"
local L_Header = "Active Quests"
local L_Label = "Quests"
local L_Complete = "Complete"
local L_Daily = "Daily"

local QG = {}
local format, tinsert, pairs, ipairs, sort, next, match, tonumber = string.format, table.insert, pairs, ipairs, table.sort, next, string.match, tonumber
QG.obj = _G.LibStub("LibDataBroker-1.1"):NewDataObject("QuestGlance", {type = "data source", text = "0/0/0", label = L_Label, icon = "Interface\\Icons\\INV_Relics_LibramofGrace"})
local MAX_QUESTS = MAX_QUESTS
_G.QuestGlance = QG

function QG.obj.OnClick()
	ToggleFrame(QuestLogFrame)
end

local frame = CreateFrame"Frame"
frame:RegisterEvent"QUEST_LOG_UPDATE"

do
	local GetNumQuestLogEntries, GetQuestLogTitle, GetQuestLogLeaderBoard, GetNumQuestLeaderBoards, CollapseQuestHeader, ExpandQuestHeader = 
		GetNumQuestLogEntries, GetQuestLogTitle, GetQuestLogLeaderBoard, GetNumQuestLeaderBoards, CollapseQuestHeader, ExpandQuestHeader
	local temp = {}
	local processing = false
	frame:SetScript("OnEvent", function(self, event)
		if processing then return end
		processing = true
		-- Find and expand any collapsed quest log entries
		local qid = 0
		local _, collapsed, name
		while true do
			qid = qid + 1
			name, _, _, _, _, collapsed = GetQuestLogTitle(qid)
			if not name then break end
			if collapsed then -- isCollapsed
				tinsert(temp, 1, qid)
				ExpandQuestHeader(qid)
			end
		end
		-- Count the number of completed quests
		local entries, quests = GetNumQuestLogEntries()
		local complete = 0
		for qid = 1,entries do
			_, _, _, _, _, _, completed = GetQuestLogTitle(qid)
			if completed == 1 then
				complete = complete + 1
			end
		end
		-- Collapse any quest headers which were collapsed before the update
		for i,qid in ipairs(temp) do
			CollapseQuestHeader(qid)
			temp[i] = nil
		end
		-- Check if values have changed
		if QG.quests ~= quests or QG.complete ~= complete then
			QG.quests, QG.complete = quests, complete
			QG:UpdateText()
		end
		-- Reset the tooltip quest cache
		QG:ResetQuestCache()
		processing = false
	end)
end
	
function QG:UpdateText()
	if showComplete and showMax then
		QG.obj.text = format("%d/%d/%d", QG.complete, QG.quests, MAX_QUESTS)
	elseif showComplete then
		QG.obj.text = format("%d/%d", QG.complete, QG.quests)
	elseif showMax then
		QG.obj.text = format("%d/%d", QG.quests, MAX_QUESTS)
	else
		QG.obj.text = tonumber(QG.quests)
	end
end
	

local function QuestSort(a, b)
	if a.complete == b.complete then
		return a.name < b.name
	else
		return b.complete
	end
end

do 
	local tCache, qCache, needsUpdate = {}, {}, true
	
	local function getT()
		local t = next(tCache)
		if t then
			tCache[t] = nil
			return t
		else
			return {}
		end
	end
	
	local function clearQuest(t)
		t.name = nil
		t.complete = nil
		t.daily = nil
		t.obj1 = nil
		t.obj2 = nil
		t.obj3 = nil
		t.obj4 = nil
	end
	
	function QG:ResetQuestCache()
		for k,v in pairs(qCache) do
			clearQuest(v)
			qCache[k] = nil
			tCache[v] = true
		end
		needsUpdate = true
	end
	
	local GetNumQuestLogEntries, GetNumQuestLeaderBoards, GetQuestLogLeaderBoard, GetQuestLogTitle =
		  GetNumQuestLogEntries, GetNumQuestLeaderBoards, GetQuestLogLeaderBoard, GetQuestLogTitle
	local temp = {}
	function QG:MakeQuestCache()
		if not needsUpdate then QG:ResetQuestCache() end
		local includeObj = false
		for i = 1, GetNumQuestLogEntries() do
			local name, _, _, _, isheader, _, complete, daily = GetQuestLogTitle(i)
			complete = complete == 1 or nil
			if not (isheader or (complete and not showCompletesInTip)) then
				if not complete then
					for j = 1, GetNumQuestLeaderBoards(i) do
						local desc,_,done = GetQuestLogLeaderBoard(j, i)
						if not done then
							tinsert(temp, desc)
						end
						if not includeObj then
							local cur, max = match(desc, "(%d+)/(%d+)$")
							includeObj = done or (cur and max and tonumber(cur) > 0)
						end
					end
				end
				if includeObj or (showCompletesInTip and complete) or (showDailiesInTip and daily) then
					local t = getT()
					tinsert(qCache, t)
					t.name = name
					t.complete = complete
					t.daily = daily
					if includeObj then
						t.obj1 = temp[1]
						t.obj2 = temp[2]
						t.obj3 = temp[3]
						t.obj4 = temp[4]
					end
				end
				includeObj = false
				for k in pairs(temp) do temp[k] = nil end
			end
		end
		sort(qCache, QuestSort)
		needsUpdate = false
	end
	
	local completeString = format("|cffffffff%s|r", L_Complete)
	local dailyString = format("|cffffffff%s|r", L_Daily)
	local clickString = format("|cffffffff%s|r", L_Click)
	function QG.obj.OnTooltipShow(tooltip)
		tooltip = tooltip or GameTooltip
		if needsUpdate then QG:MakeQuestCache() end
		tooltip:AddLine(L_Header)
		local obj
		for i, quest in ipairs(qCache) do
			if quest.complete then
				tooltip:AddDoubleLine(quest.name, completeString)
			elseif quest.daily then
				tooltip:AddDoubleLine(quest.name, dailyString)
			else
				tooltip:AddLine(quest.name)
			end
			for i=1,4 do
				obj = quest["obj"..i]
				if obj then
					tooltip:AddLine(format(" |cffffffff+%s|r", obj))
				end
			end
		end
		tooltip:AddLine(clickString)
	end
end
